@@ -328,37 +328,6 @@ FUN(PDFAnnotation_setRect)(JNIEnv *env, jobject self, jobject jrect)
 		jni_rethrow_void(env, ctx);
 }
 
-JNIEXPORT jfloat JNICALL
-FUN(PDFAnnotation_getBorder)(JNIEnv *env, jobject self)
-{
-	fz_context *ctx = get_context(env);
-	pdf_annot *annot = from_PDFAnnotation(env, self);
-	jfloat border;
-
-	if (!ctx || !annot) return 0;
-
-	fz_try(ctx)
-		border = pdf_annot_border(ctx, annot);
-	fz_catch(ctx)
-		jni_rethrow(env, ctx);
-
-	return border;
-}
-
-JNIEXPORT void JNICALL
-FUN(PDFAnnotation_setBorder)(JNIEnv *env, jobject self, jfloat border)
-{
-	fz_context *ctx = get_context(env);
-	pdf_annot *annot = from_PDFAnnotation(env, self);
-
-	if (!ctx || !annot) return;
-
-	fz_try(ctx)
-		pdf_set_annot_border(ctx, annot, border);
-	fz_catch(ctx)
-		jni_rethrow_void(env, ctx);
-}
-
 JNIEXPORT jobject JNICALL
 FUN(PDFAnnotation_getColor)(JNIEnv *env, jobject self)
 {
@@ -783,7 +752,7 @@ FUN(PDFAnnotation_update)(JNIEnv *env, jobject self)
 }
 
 JNIEXPORT jboolean JNICALL
-FUN(PDFAnnotation_isOpen)(JNIEnv *env, jobject self)
+FUN(PDFAnnotation_getIsOpen)(JNIEnv *env, jobject self)
 {
 	fz_context *ctx = get_context(env);
 	pdf_annot *annot = from_PDFAnnotation(env, self);
@@ -936,6 +905,21 @@ FUN(PDFAnnotation_setLanguage)(JNIEnv *env, jobject self, jint lang)
 		jni_rethrow_void(env, ctx);
 }
 
+JNIEXPORT jboolean JNICALL
+FUN(PDFAnnotation_hasQuadding)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	jboolean has = JNI_FALSE;
+
+	fz_try(ctx)
+		has = pdf_annot_has_quadding(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return has;
+}
+
 JNIEXPORT jint JNICALL
 FUN(PDFAnnotation_getQuadding)(JNIEnv *env, jobject self)
 {
@@ -1599,11 +1583,11 @@ FUN(PDFAnnotation_getHiddenForEditing)(JNIEnv *env, jobject self)
 }
 
 JNIEXPORT jboolean JNICALL
-FUN(PDFAnnotation_applyRedaction)(JNIEnv *env, jobject self, jboolean blackBoxes, jint imageMethod)
+FUN(PDFAnnotation_applyRedaction)(JNIEnv *env, jobject self, jboolean blackBoxes, jint imageMethod, jint lineArt)
 {
 	fz_context *ctx = get_context(env);
 	pdf_annot *annot = from_PDFAnnotation_safe(env, self);
-	pdf_redact_options opts = { blackBoxes, imageMethod };
+	pdf_redact_options opts = { blackBoxes, imageMethod, lineArt };
 	jboolean redacted = JNI_FALSE;
 
 	if (!ctx || !annot) return JNI_FALSE;
@@ -1615,3 +1599,18 @@ FUN(PDFAnnotation_applyRedaction)(JNIEnv *env, jobject self, jboolean blackBoxes
 
 	return redacted;
 }
+
+JNIEXPORT jboolean JNICALL
+FUN(PDFAnnotation_hasRect)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	jboolean has = JNI_FALSE;
+
+	fz_try(ctx)
+		has = pdf_annot_has_rect(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return has;
+}
