@@ -1,3 +1,25 @@
+// Copyright (C) 2004-2021 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
 #include "mupdf/fitz.h"
 #include "mupdf/pdf.h"
 
@@ -198,6 +220,15 @@ lex_number(fz_context *ctx, fz_stream *f, pdf_lexbuf *buf, int c)
 			isreal = s;
 			*s++ = c;
 			break;
+		case '-':
+			/* Bug 703248: Some PDFs (particularly those
+			 * generated by google docs) apparently have
+			 * numbers like 0.000000000000-5684342 in them.
+			 * We'll stop our interpretation at the -, but
+			 * keep reading to skip over the trailing
+			 * digits so they aren't parsed later. */
+			*s++ = '\0';
+			break;
 		case RANGE_0_9:
 			*s++ = c;
 			break;
@@ -288,8 +319,9 @@ lex_name(fz_context *ctx, fz_stream *f, pdf_lexbuf *lb)
 					hex[i] = lex_byte(ctx, f) - 'A' + 10;
 					break;
 				default:
-				case EOF:
 					goto illegal;
+				case EOF:
+					goto illegal_eof;
 				}
 			}
 			if (s) *s++ = (hex[0] << 4) + hex[1];
@@ -297,6 +329,7 @@ lex_name(fz_context *ctx, fz_stream *f, pdf_lexbuf *lb)
 illegal:
 			if (i == 1)
 				fz_unread_byte(ctx, f);
+illegal_eof:
 			if (s) *s++ = '#';
 			continue;
 		}
@@ -478,6 +511,7 @@ pdf_token_from_keyword(char *key)
 		break;
 	case 'n':
 		if (!strcmp(key, "null")) return PDF_TOK_NULL;
+		if (!strcmp(key, "newobj")) return PDF_TOK_NEWOBJ;
 		break;
 	case 'o':
 		if (!strcmp(key, "obj")) return PDF_TOK_OBJ;
